@@ -5,8 +5,10 @@ from serializers.serializer import (
 )
 from serializers.fields import (
     Field,
-    ModelField,
+    RelatedField,
+    PrimaryKeyRelatedField,
+    NaturalKeyRelatedField,
     ModelNameField
 )
 
-__version__ = '0.3.0'
+__version__ = '0.4.0'
