@@ -1,3 +1,4 @@
+import datetime
 from django.core.serializers.json import DateTimeAwareJSONEncoder
 from django.utils import simplejson as json
 from django.utils.encoding import smart_unicode
@@ -38,11 +39,7 @@ class YAMLRenderer(BaseRenderer):
     def render(self, obj, **opts):
         indent = opts.pop('indent', None)
         default_flow_style = opts.pop('default_flow_style', None)
-        use_ordered_dumper = opts.pop('default_flow_style', False)
-
-        dumper = use_ordered_dumper and OrderedSafeDumper or DjangoSafeDumper
-
-        return yaml.dump(obj, Dumper=dumper,
+        return yaml.dump(obj, Dumper=OrderedSafeDumper,
                          indent=indent, default_flow_style=default_flow_style)
 
 
@@ -57,36 +54,75 @@ class XMLRenderer(BaseRenderer):
 
         xml = SimplerXMLGenerator(stream, "utf-8")
         xml.startDocument()
-        xml.startElement("root", {})
-
         self._to_xml(xml, obj)
-
-        xml.endElement("root")
         xml.endDocument()
         return stream.getvalue()
 
     def _to_xml(self, xml, data):
         if isinstance(data, (list, tuple)):
             for item in data:
-                xml.startElement("list-item", {})
+                xml.startElement("item", {})
                 self._to_xml(xml, item)
-                xml.endElement("list-item")
+                xml.endElement("item")
 
         elif isinstance(data, dict):
-            # TODO: use iteritems unless sort_keys is set.
-            for key, value in sorted(data.items()):
+            xml.startElement("object", {})
+            for key, value in data.items():
                 xml.startElement(key, {})
                 self._to_xml(xml, value)
                 xml.endElement(key)
-
-        elif data is None:
-            # Don't output any value
-            pass
+            xml.endElement("object")
 
         else:
             xml.characters(smart_unicode(data))
 
 
+class DumpDataXMLRenderer(BaseRenderer):
+    """
+    Render a native python object into XML.
+    Note that this renderer is included more by way of example,
+    than as a proposed final XML renderer.
+    """
+    def render(self, obj, **opts):
+        stream = StringIO.StringIO()
+
+        xml = SimplerXMLGenerator(stream, "utf-8")
+        xml.startDocument()
+        xml.startElement("django-objects", {"version": "1.0"})
+        if isinstance(obj, (list, tuple)):
+            [self.model_to_xml(xml, item) for item in obj]
+        else:
+            self.model_to_xml(xml, obj)
+        xml.endElement("django-objects")
+        xml.endDocument()
+        return stream.getvalue()
+
+    def model_to_xml(self, xml, data):
+        pk = unicode(data['pk'])
+        model = data['model']
+        fields = data['fields']
+        xml.startElement("object", {'pk': pk, 'model': model})
+
+        # Due to implmentation details, the existing xml dumpdata format
+        # renders ordered fields, whilst json and yaml render unordered
+        # fields (ordering determined by `dict`)
+        # To maintain byte-for-byte backwards compatability,
+        # we'll deal with that now.
+        sorted_items = sorted(fields.items_with_metadata(),
+                              key=lambda x: x[2].creation_counter)
+
+        for key, value, field in sorted_items:
+            attrs = {'name': key}
+            attrs.update(field.attributes())
+            xml.startElement('field', attrs)
+            if isinstance(value, (datetime.datetime, datetime.date, datetime.time)):
+                xml.characters(value.isoformat())
+            elif value is not None:
+                xml.characters(smart_unicode(value))
+            xml.endElement('field')
+        xml.endElement("object")
+
+
 class CSVRenderer(BaseRenderer):
     def render(self, obj, **opts):
         if not hasattr(obj, '__iter__'):
