@@ -3,7 +3,7 @@ from django.core import serializers
 from django.db import models
 from django.test import TestCase
 from serializers import Serializer, ModelSerializer, DumpDataSerializer
-from serializers.fields import Field
+from serializers.fields import Field, NaturalKeyRelatedField
 
 
 class ExampleObject(object):
@@ -56,7 +56,7 @@ class EncoderTests(TestCase):
         self.assertEquals(output, expected)
 
     def test_xml(self):
-        expected = '<?xml version="1.0" encoding="utf-8"?>\n<root><a>1</a><b>foo</b><c>True</c></root>'
+        expected = '<?xml version="1.0" encoding="utf-8"?>\n<object><a>1</a><b>foo</b><c>True</c></object>'
         output = Serializer().encode(self.obj, 'xml')
         self.assertEquals(output, expected)
 
@@ -536,7 +536,7 @@ class RaceEntry(models.Model):
 class TestSimpleModel(TestCase):
     def setUp(self):
         self.dumpdata = DumpDataSerializer()
-        self.serializer = Serializer(depth=0)
+        self.serializer = ModelSerializer(depth=0)
         RaceEntry.objects.create(
             name='John doe',
             runner_number=6014,
@@ -552,8 +552,8 @@ class TestSimpleModel(TestCase):
 
     def test_csv(self):
         expected = (
-            "finish_time,start_time,id,name,runner_number\r\n"
-            "2012-04-30 12:25:00,2012-04-30 09:00:00,1,John doe,6014\r\n"
+            "id,name,runner_number,start_time,finish_time\r\n"
+            "1,John doe,6014,2012-04-30 09:00:00,2012-04-30 12:25:00\r\n"
         )
         self.assertEquals(
             self.serializer.encode(RaceEntry.objects.all(), 'csv'),
@@ -561,6 +561,144 @@ class TestSimpleModel(TestCase):
         )
 
 
+##### Model Inheritance #####
+
+class Account(models.Model):
+    points = models.PositiveIntegerField()
+    company = models.CharField(max_length=100)
+
+
+class PremiumAccount(Account):
+    date_upgraded = models.DateTimeField()
+
+
+class TestModelInheritance(TestCase):
+    def setUp(self):
+        self.dumpdata = DumpDataSerializer()
+        self.serializer = ModelSerializer()
+        PremiumAccount.objects.create(
+            points=42,
+            company='Foozle Inc.',
+            date_upgraded=datetime.datetime(year=2012, month=4, day=30, hour=9)
+        )
+
+    def test_dumpdata_child_model(self):
+        self.assertEquals(
+            self.dumpdata.encode(PremiumAccount.objects.all(), 'json'),
+            serializers.serialize('json', PremiumAccount.objects.all())
+        )
+
+    def test_serialize_child_model(self):
+        expected = [{
+            'id': 1,
+            'points': 42,
+            'company': 'Foozle Inc.',
+            'date_upgraded': datetime.datetime(2012, 4, 30, 9, 0)
+        }]
+        self.assertEquals(
+            self.serializer.serialize(PremiumAccount.objects.all()),
+            expected
+        )
+
+
+# ##### Natural Keys #####
+
+class PetOwner(models.Model):
+    first_name = models.CharField(max_length=100)
+    last_name = models.CharField(max_length=100)
+    birthdate = models.DateField()
+
+    def natural_key(self):
+        return (self.first_name, self.last_name)
+
+    class Meta:
+        unique_together = (('first_name', 'last_name'),)
+
+
+class Pet(models.Model):
+    name = models.CharField(max_length=100)
+    owner = models.ForeignKey(PetOwner, related_name='pets')
+
+    def natural_key(self):
+        return self.name
+
+
+class TestNaturalKey(TestCase):
+    """
+    Test one-to-one field relationship on a model.
+    """
+    def setUp(self):
+        self.dumpdata = DumpDataSerializer()
+        joe = PetOwner.objects.create(
+            first_name='joe',
+            last_name='adams',
+            birthdate=datetime.date(year=1965, month=8, day=27)
+        )
+        Pet.objects.create(
+            owner=joe,
+            name='splash gordon'
+        )
+        Pet.objects.create(
+            owner=joe,
+            name='frogger'
+        )
+
+    def test_naturalkey_dumpdata(self):
+        """
+        Ensure that we can replicate the existing dumpdata
+        'use_natural_keys' behaviour.
+        """
+        self.assertEquals(
+            self.dumpdata.encode(Pet.objects.all(), 'json', use_natural_keys=True),
+            serializers.serialize('json', Pet.objects.all(), use_natural_keys=True)
+        )
+
+    def test_naturalkey(self):
+        """
+        Ensure that we can use NaturalKeyRelatedField to represent foreign
+        key relationships.
+        """
+        serializer = ModelSerializer(
+            related_field=NaturalKeyRelatedField,
+            depth=0
+        )
+        expected = [{
+            "owner": (u"joe", u"adams"),  # NK, not PK
+            "id": 1,
+            "name": u"splash gordon"
+        }, {
+            "owner": (u"joe", u"adams"),  # NK, not PK
+            "id": 2,
+            "name": u"frogger"
+        }]
+        self.assertEquals(
+            serializer.serialize(Pet.objects.all()),
+            expected
+        )
+
+    def test_naturalkey_reverse_relation(self):
+        """
+        Ensure that we can use NaturalKeyRelatedField to represent
+        reverse foreign key relationships.
+        """
+        serializer = ModelSerializer(
+            include=('pets',),
+            related_field=NaturalKeyRelatedField,
+            depth=0
+        )
+        expected = [{
+            "first_name": u"joe",
+            "last_name": u"adams",
+            "id": 1,
+            "birthdate": datetime.date(1965, 8, 27),
+            "pets": [u"splash gordon", u"frogger"]  # NK, not PK
+        }]
+        self.assertEquals(
+            serializer.serialize(PetOwner.objects.all()),
+            expected
+        )
+
+
 ##### One to one relationships #####
 
 class User(models.Model):
@@ -600,6 +738,12 @@ class TestOneToOneModel(TestCase):
             serializers.serialize('yaml', Profile.objects.all())
         )
 
+    def test_onetoone_dumpdata_xml(self):
+        self.assertEquals(
+            self.dumpdata.encode(Profile.objects.all(), 'xml'),
+            serializers.serialize('xml', Profile.objects.all())
+        )
+
     def test_onetoone_nested(self):
         expected = {
             'id': 1,
@@ -719,6 +863,12 @@ class TestFKModel(TestCase):
             serializers.serialize('yaml', Vehicle.objects.all())
         )
 
+    def test_fk_dumpdata_xml(self):
+        self.assertEquals(
+            self.dumpdata.encode(Vehicle.objects.all(), 'xml'),
+            serializers.serialize('xml', Vehicle.objects.all())
+        )
+
     def test_fk_nested(self):
         expected = {
             'id': 1,
@@ -841,6 +991,20 @@ class TestManyToManyModel(TestCase):
             serializers.serialize('yaml', Author.objects.all())
         )
 
+    # TODO
+    # def test_m2m_dumpdata_xml(self):
+    #     # # Hack to ensure field ordering is correct for xml
+    #     # dumpdata = DumpDataSerializer()
+    #     # dumpdata.fields['fields'].opts.preserve_field_order = True
+    #     self.assertEquals(
+    #         self.dumpdata.encode(Book.objects.all(), 'xml'),
+    #         serializers.serialize('xml', Book.objects.all())
+    #     )
+    #     self.assertEquals(
+    #         self.dumpdata.encode(Author.objects.all(), 'xml'),
+    #         serializers.serialize('xml', Author.objects.all())
+    #     )
+
     def test_m2m_nested(self):
         expected = {
             'id': 1,
