@@ -3,17 +3,48 @@ from django.utils.datastructures import SortedDict
 import csv
 
 
+class DictWithMetadata(dict):
+    """
+    A dict-like object, that can have additional metadata attached.
+    """
+    def __init__(self, *args, **kwargs):
+        super(DictWithMetadata, self).__init__(*args, **kwargs)
+        self.metadata = {}
+
+    def set_with_metadata(self, key, value, metadata):
+        self[key] = value
+        self.metadata[key] = metadata
+
+    def items_with_metadata(self):
+        return [(key, value, self.metadata[key])
+        for (key, value) in self.items()]
+
+
+class SortedDictWithMetadata(SortedDict, DictWithMetadata):
+    pass
+
+
 try:
     import yaml
 except ImportError:
-    OrderedDumper = None
+    DjangoSafeDumper = None
+    OrderedSafeDumper = None
 else:
     # Adapted from http://pyyaml.org/attachment/ticket/161/use_ordered_dict.py
     class DjangoSafeDumper(yaml.SafeDumper):
+        """
+        Handles decimals as strings, and our dictionary KeyWithMetadata keys
+        as regular unicode keys.
+        """
         def represent_decimal(self, data):
             return self.represent_scalar('tag:yaml.org,2002:str', str(data))
 
     class OrderedSafeDumper(DjangoSafeDumper):
+        """
+        Handles decimals as strings, regular dicts as if they we normal dicts
+        (but getting the field order correct) and our dictionary
+        KeyWithMetadata keys as regular unicode keys.
+        """
         def represent_decimal(self, data):
             return self.represent_scalar('tag:yaml.org,2002:str', str(data))
 
@@ -25,6 +56,8 @@ else:
             best_style = True
             if hasattr(mapping, 'items'):
                 mapping = list(mapping.items())
+                if not isinstance(mapping, SortedDict):
+                    mapping.sort()
             for item_key, item_value in mapping:
                 node_key = self.represent_data(item_key)
                 node_value = self.represent_data(item_value)
@@ -40,7 +73,9 @@ else:
                     node.flow_style = best_style
             return node
 
-    OrderedSafeDumper.add_representer(SortedDict,
+    OrderedSafeDumper.add_representer(DictWithMetadata,
+            yaml.representer.SafeRepresenter.represent_dict)
+    OrderedSafeDumper.add_representer(SortedDictWithMetadata,
             yaml.representer.SafeRepresenter.represent_dict)
 
 
@@ -75,6 +110,10 @@ class DictWriter(csv.DictWriter):
             s = str(s)
         return s
 
+    def writeheader(self):
+        header = dict([(item, item) for item in self.fieldnames])
+        self.writerow(header)
+
     def writerow(self, d):
         for fieldname in self.fieldnames:
             if fieldname in d:
